using System;
using Microsoft.SPOT.Hardware;
using GHIElectronics.NETMF.FEZ;
using GHIElectronics.NETMF.Hardware;

namespace ORG.MUJIRUSHI.NETMF
{
    public class BMP085 : IDisposable
    {
        const ushort BMP085_ADDRESS = 0x77;
        const int CLOCK_FREQ = 400;
        const int TIMEOUT = 1000;
        const byte OSS = 0;

        I2CDevice i2c;

        short ac1, ac2, ac3;
        ushort ac4, ac5, ac6;
        short b1, b2;
        short mb, mc, md;

        int ut;
        int up;

        public BMP085()
        {
            i2c = new I2CDevice(new I2CDevice.Configuration(BMP085_ADDRESS, CLOCK_FREQ));

            System.Threading.Thread.Sleep(10);

            ac1 = (short)(read16(0xAA));
            ac2 = (short)(read16(0xAC));
            ac3 = (short)(read16(0xAE));
            ac4 = (ushort)(read16(0xB0));
            ac5 = (ushort)(read16(0xB2));
            ac6 = (ushort)(read16(0xB4));
            b1 = (short)(read16(0xB6));
            b2 = (short)(read16(0xB8));
            mb = (short)(read16(0xBA));
            mc = (short)(read16(0xBC));
            md = (short)(read16(0xBE));
        }

        public void Dispose()
        {
            i2c.Dispose();
            GC.SuppressFinalize(this);
        }

        public void Measurement()
        {
            write8(0xF4, 0x2E);
            System.Threading.Thread.Sleep(5);
            ut = read16(0xF6);

            write8(0xF4, (byte)(0x34 | (OSS << 6)));
            System.Threading.Thread.Sleep(2 + (3 << OSS));
            up = (int)read24(0xF6) >> (8 - OSS);
        }

        public double GetPressure()
        {
            int x1, x2, x3;
            int b3;
            uint b4;
            int b5;
            int b6;
            uint b7;
            int p;

            x1 = ((ut - ac6) * ac5) >> 15;
            x2 = (mc << 11) / (x1 + md);
            b5 = x1 + x2;
  
            b6 = b5 - 4000;

            x1 = (b2 * ((b6 * b6) >> 12)) >> 11;
            x2 = (ac2 * b6) >> 11;
            x3 = x1 + x2;
            b3 = ((((ac1) * 4 + x3) << OSS) + 2) / 4;
  
            x1 = (ac3 * b6) >> 13;
            x2 = (b1 * ((b6 * b6) >> 12)) >> 16;
            x3 = ((x1 + x2) + 2) >> 2;
            b4 = (ac4 * (uint)(x3 + 32768)) >> 15;

            b7 = (uint)((up - b3) * (50000 >> OSS));
            if (b7 < 0x80000000)
                p = (int)((b7 * 2) / b4);
            else
                p = (int)((b7 / b4) * 2);
    
            x1 = (p >> 8) * (p >> 8);
            x1 = (x1 * 3038) >> 16;
            x2 = (-7357 * p) >> 16;
            p += (x1 + x2 + 3791) >> 4;

            return p;
        }

        public double GetTemperatureCelsius()
        {
            int x1, x2;
            int b5;
            int t;

            x1 = ((ut - ac6) * ac5) >> 15;
            x2 = (mc << 11) / (x1 + md);
            b5 = x1 + x2;
            t = (b5 + 8) >> 4;

            return t / 10.0;
        }

        private void write8(byte addr, byte data)
        {
            i2c.Execute(new I2CDevice.I2CTransaction[] {
                    I2CDevice.CreateWriteTransaction(new byte[] { addr, data }) }, TIMEOUT);
        }

        private byte read8(byte addr)
        {
            byte[] buffer = new byte[1];
            i2c.Execute(new I2CDevice.I2CTransaction[] {
                    I2CDevice.CreateWriteTransaction(new byte[] { addr }),
                    I2CDevice.CreateReadTransaction(buffer) }, TIMEOUT);
            return buffer[0];
        }

        private ushort read16(byte addr)
        {
            byte[] buffer = new byte[2];
            i2c.Execute(new I2CDevice.I2CTransaction[] {
                    I2CDevice.CreateWriteTransaction(new byte[] { addr }),
                    I2CDevice.CreateReadTransaction(buffer) }, TIMEOUT);
            return (ushort)((buffer[0] << 8) | buffer[1]);
        }

        private uint read24(byte addr)
        {
            byte[] buffer = new byte[3];
            i2c.Execute(new I2CDevice.I2CTransaction[] {
                    I2CDevice.CreateWriteTransaction(new byte[] { addr }),
                    I2CDevice.CreateReadTransaction(buffer) }, TIMEOUT);
            return (uint)((buffer[0] << 16) | (buffer[1] << 8) | buffer[2]);
        }
    }
}