﻿using System;
using System.Threading;
using Microsoft.SPOT;
using Microsoft.SPOT.Hardware;
using Microsoft.SPOT.Input;
using Microsoft.SPOT.Net.NetworkInformation;
using Microsoft.SPOT.Presentation;
using Microsoft.SPOT.Presentation.Controls;
using GHIElectronics.NETMF.FEZ;
using Komodex.NETMF.MicroTweet;
using ORG.MUJIRUSHI.NETMF;

namespace FEZCobraNETMFMicroTweet
{
    public class MainWindow : Window
    {
        Panel panel = new Panel();
        Image wallpaperImage = new Image();
        Panel panelText = new Panel();
        StackPanel stackText = new StackPanel();
        Text pressureText = new Text();
        Text temperatureText = new Text();
        Text humidityText = new Text();
        Panel panelImage = new Panel();
        Image weatherImage = new Image();

        public MainWindow()
        {
            weatherImage.Bitmap = null;
            weatherImage.HorizontalAlignment = HorizontalAlignment.Center;
            weatherImage.VerticalAlignment = VerticalAlignment.Center;

            panelImage.Children.Add(weatherImage);
            panelImage.HorizontalAlignment = HorizontalAlignment.Right;
            panelImage.VerticalAlignment = VerticalAlignment.Bottom;
            panelImage.Width = SystemMetrics.ScreenWidth / 4;
            panelImage.Height = SystemMetrics.ScreenHeight / 4;

            pressureText.Font = Resources.GetFont(Resources.FontResources.Msyh24);
            pressureText.HorizontalAlignment = HorizontalAlignment.Center;
            pressureText.VerticalAlignment = VerticalAlignment.Center;

            temperatureText.Font = Resources.GetFont(Resources.FontResources.Msyh24);
            temperatureText.HorizontalAlignment = HorizontalAlignment.Center;
            temperatureText.VerticalAlignment = VerticalAlignment.Center;

            humidityText.Font = Resources.GetFont(Resources.FontResources.Msyh24);
            humidityText.HorizontalAlignment = HorizontalAlignment.Center;
            humidityText.VerticalAlignment = VerticalAlignment.Center;

            stackText.Children.Add(pressureText);
            stackText.Children.Add(temperatureText);
            stackText.Children.Add(humidityText);
            stackText.Orientation = Orientation.Vertical;
            stackText.HorizontalAlignment = HorizontalAlignment.Center;
            stackText.VerticalAlignment = VerticalAlignment.Center;

            panelText.Children.Add(stackText);
            panelText.HorizontalAlignment = HorizontalAlignment.Right;
            panelText.VerticalAlignment = VerticalAlignment.Top;
            panelText.Width = SystemMetrics.ScreenWidth / 2;
            panelText.Height = SystemMetrics.ScreenHeight / 2;

            wallpaperImage.Bitmap = Resources.GetBitmap(Resources.BitmapResources.robot);
            wallpaperImage.HorizontalAlignment = HorizontalAlignment.Stretch;
            wallpaperImage.VerticalAlignment = VerticalAlignment.Stretch;

            panel.Children.Add(wallpaperImage);
            panel.Children.Add(panelText);
            panel.Children.Add(panelImage);
            panel.HorizontalAlignment = HorizontalAlignment.Stretch;
            panel.VerticalAlignment = VerticalAlignment.Stretch;

            this.Child = panel;
            this.Width = SystemMetrics.ScreenWidth;
            this.Height = SystemMetrics.ScreenHeight;
        }

        int interval = 3600;    /* 1hour */
        BMP085 pressureDevice;
        HIH5031 humidityDevice;
        TwitterClient twitterClient;
        System.Threading.Thread measurementThread;

        protected override void OnGotFocus(FocusChangedEventArgs e)
        {
            base.OnGotFocus(e);

            pressureDevice = new BMP085();
            humidityDevice = new HIH5031(FEZ_Pin.AnalogIn.AD7);

            //NetworkInterface[] networks = NetworkInterface.GetAllNetworkInterfaces();
            //if (networks[0].IsDhcpEnabled)
            //{
            //    networks[0].RenewDhcpLease();
            //}

            twitterClient = new TwitterClient(Config.ConsumerKey,
                    Config.ConsumerSecret, Config.UserToken, Config.UserTokenSecret);
            twitterClient.DebugMessage += new DebugMessageEventHandler(twitterClient_DebugMessage);

            NTP.UpdateTimeFromNTPServer(Config.NTPServer);

            measurementThread = new System.Threading.Thread(MeasurementThread);
            measurementThread.Start();
        }

        void MeasurementThread()
        {
            double pressure, oldpressure;
            double temperature;
            double humidity;
            string message;

            pressureDevice.Measurement();
            oldpressure = System.Math.Round(pressureDevice.GetPressure() / 100 * 10) / 10;

            while (true)
            {
                pressureDevice.Measurement();

                pressure = System.Math.Round(pressureDevice.GetPressure() / 100 * 10) / 10;
                Dispatcher.BeginInvoke(
                        new DispatcherOperationCallback(PressureText), pressure.ToString("F1") + "hPa");
                message = pressure.ToString("F1") + "hPa";

                temperature = System.Math.Round(pressureDevice.GetTemperatureCelsius() * 10) / 10;
                Dispatcher.BeginInvoke(
                        new DispatcherOperationCallback(TemperatureText), temperature.ToString("F1") + "\u00B0C");
                message += " " + temperature.ToString("F1") + "C";

                humidity = System.Math.Round(humidityDevice.GetHumidity(temperature) * 10) / 10;
                Dispatcher.BeginInvoke(
                        new DispatcherOperationCallback(HumidityText), humidity.ToString("F1") + "%");
                message += " " + humidity.ToString("F1") + "%";

                if (0.5 < (pressure - oldpressure))         /* Stable good weather(50Pa/h < dP/dt) */
                {
                    Dispatcher.BeginInvoke(
                            new DispatcherOperationCallback(WeatherImage),
                                    Resources.GetBitmap(Resources.BitmapResources.good));
                    message += " good";
                }
                else if ((pressure - oldpressure) < -0.5)    /* Stable rainy weather(dP/dt < -50Pa/h) */
                {
                    Dispatcher.BeginInvoke(
                            new DispatcherOperationCallback(WeatherImage),
                                    Resources.GetBitmap(Resources.BitmapResources.rainy));
                    message += " rainy";
                }
                else
                {
                    Dispatcher.BeginInvoke(
                            new DispatcherOperationCallback(WeatherImage), null);
                }
                oldpressure = pressure;

                //Debug.Print(message);
                twitterClient.SendTweet(message);

                Thread.Sleep(interval * 1000);
            }
        }

        object PressureText(object obj)
        {
            pressureText.TextContent = obj as string;
            return null;
        }

        object TemperatureText(object obj)
        {
            temperatureText.TextContent = obj as string;
            return null;
        }

        object HumidityText(object obj)
        {
            humidityText.TextContent = obj as string;
            return null;
        }

        object WeatherImage(object obj)
        {
            weatherImage.Bitmap = obj as Bitmap;
            return null;
        }

        void twitterClient_DebugMessage(object sender, DebugMessageEventArgs e)
        {
            Debug.Print(e.Message);
        }
    }
}
